@@ -15,13 +15,46 @@ def attributes(**kwargs):
     return inner
 
 
-class Everything:
+class _Everything:
     def __contains__(self, item):
         return True
 
 
-class AdminMixin:
-    def get_fields_for_request(self, request):
+class _AdminOptions:
+    ddb_ignore = False
+    ddb_extra_fields = []
+    ddb_hide_fields = []
+    ddb_json_fields = {}
+    ddb_default_filters = []
+
+    def get_ddb_ignore(self, request):
+        return self.ddb_ignore
+
+    def get_ddb_extra_fields(self, request):
+        return self.ddb_extra_fields
+
+    def get_ddb_hide_fields(self, request):
+        return self.ddb_hide_fields
+
+    def get_ddb_json_fields(self, request):
+        return self.ddb_json_fields
+
+    def get_ddb_default_filters(self):
+        return self.ddb_default_filters
+
+
+def _get_option(admin, name, *args):
+    field = f"ddb_{name}"
+    func = f"get_ddb_{name}"
+
+    if hasattr(admin, func):
+        return getattr(admin, func)(*args)
+    else:
+        return getattr(admin, field, getattr(_AdminOptions, field))
+
+
+class _AdminAnnotations:
+    def _get_fields_for_request(self, request):
         if hasattr(request, "data_browser"):
             return request.data_browser["fields"]
         elif (
@@ -30,11 +63,11 @@ class AdminMixin:
         ):
             return set(self.get_list_display(request))
         else:
-            return Everything()
+            return _Everything()
 
     def get_queryset(self, request):
         qs = super().get_queryset(request)
-        fields = self.get_fields_for_request(request)
+        fields = self._get_fields_for_request(request)
 
         for name, descriptor in self._ddb_annotations().items():
             if name not in fields:
@@ -67,15 +100,18 @@ class AdminMixin:
             cls._ddb_annotations_real = {}
         return cls._ddb_annotations_real
 
+
+class AdminMixin(_AdminOptions, _AdminAnnotations):
     def changelist_view(self, request, extra_context=None):
+        """ Inject ddb_url """
         extra_context = extra_context or {}
 
-        if not getattr(self, "ddb_ignore", False):
+        if not self.get_ddb_ignore(request):
             url = reverse(
                 "data_browser:query_html",
                 args=[f"{self.model._meta.app_label}.{self.model.__name__}", ""],
             )
-            args = getattr(self, "ddb_default_filters", [])
+            args = self.get_ddb_default_filters()
             params = urlencode(
                 [
                     (
@@ -90,7 +126,7 @@ class AdminMixin:
         return super().changelist_view(request, extra_context)
 
 
-class AnnotationDescriptor:
+class _AnnotationDescriptor:
     def __init__(self, get_queryset):
         self.get_queryset = get_queryset
 
@@ -122,4 +158,4 @@ def ddb_hide(func):  # pragma: no cover
     return func
 
 
-annotation = AnnotationDescriptor
+annotation = _AnnotationDescriptor
