@@ -1,6 +1,9 @@
+import json
 from collections import defaultdict
+from dataclasses import dataclass
 
 from django.contrib.admin import site
+from django.contrib.admin.options import BaseModelAdmin
 from django.contrib.admin.utils import flatten_fieldsets
 from django.contrib.auth.admin import UserAdmin
 from django.db import models
@@ -11,72 +14,56 @@ from django.utils.html import format_html
 from django.utils.text import slugify
 
 from .common import debug_log, settings
-from .helpers import AdminMixin, AnnotationDescriptor, attributes
+from .helpers import AdminMixin, _AnnotationDescriptor, _get_option, attributes
+from .orm_aggregates import get_aggregates_for_type
 from .orm_fields import (
     OrmAnnotatedField,
     OrmCalculatedField,
     OrmConcreteField,
     OrmFileField,
     OrmFkField,
-    OrmModel,
-    get_fields_for_type,
-    get_model_name,
+    OrmRawField,
 )
-from .types import (
-    TYPES,
-    BooleanType,
-    DateTimeType,
-    DateType,
-    DurationType,
-    JSONType,
-    NumberArrayType,
-    NumberChoiceArrayType,
-    NumberChoiceType,
-    NumberType,
-    StringArrayType,
-    StringChoiceArrayType,
-    StringChoiceType,
-    StringType,
-    UnknownType,
-)
-
-try:
-    from django.contrib.postgres.fields import ArrayField
-except ModuleNotFoundError:  # pragma: postgres
-    ArrayField = None.__class__
-
-try:
-    from django.db.models import JSONField
-except ImportError:  # pragma: django < 3.1
-    try:
-        from django.contrib.postgres.fields import JSONField
-    except ModuleNotFoundError:  # pragma: postgres
-        JSONField = None.__class__
-
+from .orm_functions import get_functions_for_type
+from .orm_types import get_field_type
+from .types import TYPES, BooleanType, JSONType, NumberType, StringType
 
 OPEN_IN_ADMIN = "admin"
 
-_STRING_FIELDS = (
-    models.CharField,
-    models.TextField,
-    models.GenericIPAddressField,
-    models.UUIDField,
-)
-_NUMBER_FIELDS = (
-    models.DecimalField,
-    models.FloatField,
-    models.IntegerField,
-    models.AutoField,
-)
-_FIELD_TYPE_MAP = {
-    models.BooleanField: BooleanType,
-    models.DurationField: DurationType,
-    models.NullBooleanField: BooleanType,
-    models.DateTimeField: DateTimeType,
-    models.DateField: DateType,
-    **{f: StringType for f in _STRING_FIELDS},
-    **{f: NumberType for f in _NUMBER_FIELDS},
-}
+
+@dataclass
+class OrmModel:
+    fields: dict
+    admin: BaseModelAdmin = None
+
+    @property
+    def root(self):
+        return bool(self.admin)
+
+    @property
+    def default_filters(self):
+        default_filters = _get_option(self.admin, "default_filters")
+        assert isinstance(default_filters, list)
+        return [
+            (f, l, v if isinstance(v, str) else json.dumps(v))
+            for (f, l, v) in default_filters
+        ]
+
+
+def get_model_name(model, sep="."):
+    return f"{model._meta.app_label}{sep}{model.__name__}"
+
+
+def get_fields_for_type(type_):
+    aggregates = get_aggregates_for_type(type_)
+    functions = get_functions_for_type(type_)
+    others = {}
+    if type_.raw_type:
+        others["raw"] = OrmRawField(
+            type_.name, "raw", "raw", type_.raw_type, type_.raw_type.name, None
+        )
+
+    return {**aggregates, **functions, **others}
 
 
 @attributes(short_description=OPEN_IN_ADMIN)
@@ -118,10 +105,13 @@ def _get_all_admin_fields(request):
                 f"{type(model_admin)} instance does not look like a ModelAdmin or InlineModelAdmin"
             )
             return False
-        if getattr(model_admin, "ddb_ignore", False):
+
+        if _get_option(model_admin, "ignore", request):
             return False
+
         if model_admin.has_change_permission(request):
             return True
+
         if hasattr(model_admin, "has_view_permission"):
             return model_admin.has_view_permission(request)
         else:
@@ -129,15 +119,19 @@ def _get_all_admin_fields(request):
 
     all_admin_fields = defaultdict(set)
     hidden_fields = defaultdict(set)
+
+    def get_common(admin, model):
+        all_admin_fields[model].update(from_fieldsets(admin, False))
+        all_admin_fields[model].update(_get_option(admin, "extra_fields", request))
+        hidden_fields[model].update(_get_option(admin, "hide_fields", request))
+
     model_admins = {}
     for model, model_admin in site._registry.items():
         if visible(model_admin, request):
             model_admins[model] = model_admin
-            all_admin_fields[model].update(from_fieldsets(model_admin, False))
             all_admin_fields[model].update(model_admin.get_list_display(request))
-            all_admin_fields[model].update(getattr(model_admin, "ddb_extra_fields", []))
             all_admin_fields[model].add(open_in_admin)
-            hidden_fields[model].update(getattr(model_admin, "ddb_hide_fields", []))
+            get_common(model_admin, model)
 
             # check the inlines, these are already filtered for access
             for inline in model_admin.get_inline_instances(request):
@@ -147,19 +141,10 @@ def _get_all_admin_fields(request):
                     except Exception as e:
                         debug_log(e)  # ignore things like GenericInlineModelAdmin
                     else:
-
                         if inline.model not in model_admins:  # pragma: no branch
                             model_admins[inline.model] = inline
-                        all_admin_fields[inline.model].update(
-                            from_fieldsets(inline, False)
-                        )
-                        all_admin_fields[inline.model].update(
-                            getattr(inline, "ddb_extra_fields", [])
-                        )
                         all_admin_fields[inline.model].add(fk_field.name)
-                        hidden_fields[inline.model].update(
-                            getattr(inline, "ddb_hide_fields", [])
-                        )
+                        get_common(inline, inline.model)
 
     for model, model_admin in model_admins.items():
         if isinstance(model_admin, AdminMixin):
@@ -201,7 +186,7 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
         getattr(field_func, "short_description", field_name.replace("_", " "))
     )
 
-    if isinstance(field_func, AnnotationDescriptor):
+    if isinstance(field_func, _AnnotationDescriptor):
         qs = admin_get_queryset(admin, request, [field_name])
 
         annotation = qs.query.annotations.get(field_name)
@@ -210,19 +195,19 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
                 f"Can't find annotation '{field_name}' for {admin}.{field_name}"
             )
 
-        field_type = getattr(annotation, "output_field", None)
-        if not field_type:  # pragma: no cover
+        field = getattr(annotation, "output_field", None)
+        if not field:  # pragma: no cover
             raise Exception(
                 f"Annotation '{field_name}' for {admin}.{field_name} doesn't specify 'output_field'"
             )
 
-        type_, choices = _get_field_type(model, field_name, field_type)
+        type_, choices = get_field_type(field)
         return OrmAnnotatedField(
             model_name=model_name,
             name=field_name,
             pretty_name=pretty_name,
             type_=type_,
-            field_type=field_type,
+            django_field=field,
             admin=admin,
             choices=choices,
         )
@@ -241,60 +226,6 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
         )
 
 
-def _fmt_choices(choices):
-    return [(value, str(label)) for value, label in choices or []]
-
-
-def _get_field_type(model, field_name, field):
-    if isinstance(field, ArrayField) and isinstance(
-        field.base_field, _STRING_FIELDS
-    ):  # pragma: postgres
-        base_field, choices = _get_field_type(model, field_name, field.base_field)
-        array_types = {
-            StringType: StringArrayType,
-            NumberType: NumberArrayType,
-            StringChoiceType: StringChoiceArrayType,
-            NumberChoiceType: NumberChoiceArrayType,
-        }
-        if base_field in array_types:
-            return array_types[base_field], choices
-        else:
-            debug_log(
-                f"{model.__name__}.{field_name} unsupported subarray type {type(field.base_field).__name__}"
-            )
-            return UnknownType, None
-
-    elif isinstance(field, ArrayField) and isinstance(
-        field.base_field, _NUMBER_FIELDS
-    ):  # pragma: postgres
-        if field.base_field.choices:
-            return NumberChoiceArrayType, _fmt_choices(field.base_field.choices)
-        else:
-            return NumberArrayType, None
-    elif isinstance(field, JSONField):
-        res = JSONType
-    elif field.__class__ in _FIELD_TYPE_MAP:
-        res = _FIELD_TYPE_MAP[field.__class__]
-    else:
-        for django_type, field_type in _FIELD_TYPE_MAP.items():
-            if isinstance(field, django_type):
-                res = field_type
-                break
-        else:
-            debug_log(
-                f"{model.__name__}.{field_name} unsupported type {type(field).__name__}"
-            )
-            res = UnknownType
-
-    # Choice fields have different lookups
-    if res is StringType and field.choices:
-        return StringChoiceType, _fmt_choices(field.choices)
-    elif res is NumberType and field.choices:
-        return NumberChoiceType, _fmt_choices(field.choices)
-    else:
-        return res, None
-
-
 def _make_json_sub_module(model_name, field_types):
     TYPE_MAP = {"string": StringType, "number": NumberType, "boolean": BooleanType}
 
@@ -354,11 +285,11 @@ def _get_fields_for_model(request, model, admin, admin_fields):
                 fields[orm_field.name] = orm_field
         # Normal fields
         else:
-            field_type, choices = _get_field_type(model, field_name, field)
+            field_type, choices = get_field_type(field)
 
             rel_name = field_type.name
             if field_type is JSONType:
-                json_fields = getattr(admin, "ddb_json_fields", {}).get(field_name)
+                json_fields = _get_option(admin, "json_fields", request).get(field_name)
                 if json_fields:  # pragma: no branch
                     rel_name = f"{model_name}__{field_name}"
                     orm_models[rel_name] = _make_json_sub_module(
