@@ -49,6 +49,9 @@ class OrmBoundField:
     def __getattr__(self, name):
         return getattr(self.field, name)
 
+    def parse(self, lookup, value):
+        return self.type_.parse(lookup, value, self.choices)
+
     @classmethod
     def blank(cls):
         return cls(
