@@ -65,13 +65,26 @@ def test_query_query(admin_client, snapshot):
 
 @pytest.mark.parametrize("format", ["sql", "profile", "pstats", "profile_sql"])
 def test_query_misc_formats(admin_client, format):
-    # we're not going to check the result as they vary it's sufficient that it doesn't blow up
+    # we're not going to check the result as they vary and it's sufficient that it doesn't blow up
     res = admin_client.get(
         f"/data_browser/query/core.Product/size-0,name+1,size_unit.{format}?size__lt=2&id__gt=0"
     )
     assert res.status_code == 200
 
 
+@pytest.mark.skipif(django.VERSION < (2, 1), reason="Django version 2.1 required")
+def test_query_explain(admin_client):
+    res = admin_client.get(
+        f"/data_browser/query/core.Product/size-0,name+1,size_unit.explain?size__lt=2&id__gt=0"
+    )
+    assert res.status_code == 200
+
+
+def test_query_sql_aggregate(admin_client):
+    res = admin_client.get(f"/data_browser/query/core.Product/size__count.sql")
+    assert res.status_code == 200
+
+
 @pytest.mark.parametrize(
     "format", ["bad", "profile_bad", "pstats_bad", "profilesql", "pstatsbad"]
 )
@@ -302,10 +315,11 @@ def test_view_json(admin_client):
         "formatHints": {
             "name": {},
             "size": {
-                "decimalPlaces": 0,
-                "significantFigures": 3,
+                "highCutOff": 10000000000.0,
                 "lowCutOff": 0.0001,
-                "highCutOff": 1e10,
+                "maximumFractionDigits": 0,
+                "minimumFractionDigits": 0,
+                "significantFigures": 3,
             },
             "size_unit": {},
         },
